@@ -5,23 +5,42 @@ import requests
 from .mof import Mof
 
 
-def get_page(params, page=1) -> Tuple[List[Mof], int]:
+def get_page(params, headers: Dict[str, str], page=1) -> Tuple[List[Mof], int]:
     if page and page != 1:
         params["page"] = page
-    r = requests.get('https://mof.tech.northwestern.edu/mofs.json', params=params)
+    r = requests.get('https://mof.tech.northwestern.edu/mofs.json', headers=headers, params=params)
     json_response = r.json()
     return [Mof(x) for x in json_response['results']], json_response["pages"]
 
 
-def get_all(params: Dict[str, str]) -> Generator[Mof, None, None]:
+def get_all(params: Dict[str, str], pressure_unit: str = None, loading_unit: str = None) -> Generator[Mof, None, None]:
+    headers = {}
+    if pressure_unit or loading_unit:
+        r = requests.get('https://mof.tech.northwestern.edu/classifications.json')
+        classifications = r.json()
+        pressure_units = [cls['name'] for cls in classifications if cls["type"] == "pressure"]
+        loading_units = [cls['name'] for cls in classifications if cls["type"] == "loading"]
+        if pressure_unit is not None and pressure_unit not in pressure_units:
+            raise InvalidUnit(
+                f"'{pressure_unit}' is not a valid unit for pressure. Valid pressure units are: {pressure_units}")
+        if loading_unit is not None and loading_unit not in loading_units:
+            raise InvalidUnit(
+                f"'{loading_unit}' is not a valid unit for loading. Valid loading units are: {loading_units}")
+        headers["pressure"] = pressure_unit
+        headers["loading"] = loading_unit
+
     page = 1
     pages = 2
     while page <= pages:
-        mofs, pages = get_page(params, page)
+        mofs, pages = get_page(params, headers, page)
         yield from mofs
         page += 1
 
 
+class InvalidUnit(Exception):
+    pass
+
+
 def fetch(mofid: str = None,
           mofkey: str = None,
           vf_min: float = None,
@@ -35,23 +54,28 @@ def fetch(mofid: str = None,
           sa_m2cm3_min: float = None,
           sa_m2cm3_max: float = None,
           telemetry: bool = True,
+          pressure_unit: str = None,
+          loading_unit: str = None,
           limit: int = None) -> Generator[Mof, None, None]:
     if telemetry:
         try:
             yield from fetch_inner(mofid=mofid, mofkey=mofkey, vf_min=vf_min, vf_max=vf_max, lcd_min=lcd_min,
-                               lcd_max=lcd_max, pld_min=pld_min, pld_max=pld_max, sa_m2g_min=sa_m2g_min,
-                               sa_m2g_max=sa_m2g_max, sa_m2cm3_min=sa_m2cm3_min,
-                               sa_m2cm3_max=sa_m2cm3_max, limit=limit)
+                                   lcd_max=lcd_max, pld_min=pld_min, pld_max=pld_max, sa_m2g_min=sa_m2g_min,
+                                   sa_m2g_max=sa_m2g_max, sa_m2cm3_min=sa_m2cm3_min,
+                                   sa_m2cm3_max=sa_m2cm3_max, pressure_unit=pressure_unit, loading_unit=loading_unit,
+                                   limit=limit)
         except Exception as e:
             import sentry_sdk
-            sentry_sdk.init("https://287d83a67df94a3288777a876182cfcc@o310079.ingest.sentry.io/6290292", traces_sample_rate=0.0)
+            sentry_sdk.init("https://287d83a67df94a3288777a876182cfcc@o310079.ingest.sentry.io/6290292",
+                            traces_sample_rate=0.0)
             sentry_sdk.capture_exception(e)
             raise e
     else:
         yield from fetch_inner(mofid=mofid, mofkey=mofkey, vf_min=vf_min, vf_max=vf_max, lcd_min=lcd_min,
                                lcd_max=lcd_max, pld_min=pld_min, pld_max=pld_max, sa_m2g_min=sa_m2g_min,
                                sa_m2g_max=sa_m2g_max, sa_m2cm3_min=sa_m2cm3_min,
-                               sa_m2cm3_max=sa_m2cm3_max, limit=limit)
+                               sa_m2cm3_max=sa_m2cm3_max, pressure_unit=pressure_unit, loading_unit=loading_unit,
+                               limit=limit)
 
 
 def fetch_inner(
@@ -67,6 +91,8 @@ def fetch_inner(
         sa_m2g_max: float = None,
         sa_m2cm3_min: float = None,
         sa_m2cm3_max: float = None,
+        pressure_unit: str = None,
+        loading_unit: str = None,
         limit: int = None) -> Generator[Mof, None, None]:
     params = {}
     if mofid:
@@ -98,10 +124,10 @@ def fetch_inner(
 
     if limit:
         yielded = 0
-        mofs = get_all(params)
+        mofs = get_all(params, pressure_unit=pressure_unit, loading_unit=loading_unit)
         while yielded < limit:
             yield mofs.__next__()
             yielded += 1
 
     else:
-        yield from get_all(params)
+        yield from get_all(params, pressure_unit=pressure_unit, loading_unit=loading_unit)
